/*
 *  This class unmarshals the application XML file components into JAXB 
 *  generated objects.
 *
 *  @author :: Dan Dumitru ::
 *  Created on 2003-07-14
 *	
 *	This class unmarshals the application XML file components into JAXB 
 *  generated objects.  These objects are made available to other
 *  application components via getter() methods
 * 
 *  This class is based upon the sample class presented in the
 *  tutorial "Data binding with JAXB" presented by developerWorks.
 *  Just go to http://www.ibm.com/developerWorks and search for
 *  "Data binding with JAXB"
 *
 */
 
 package gov.grants.utility;

// sun
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.File;
import java.io.InputStream;

// jwsdp - sun
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.UnmarshallerHandler;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;

// Apache
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xmlsoap.schemas.soap.envelope.*;

// Grants.gov
import gov.grants.grantapplication.*;
import gov.grants.utility.GrantsGovConstantsUtils;

public final class GrantApplicationProcessor {

	// Used to configure a validating SAX2.0 parser (Xerces2)
	static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
	static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";
	static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";
	static final String SOAP_ENVELOPE_SCHEMA = "http://localhost:8080/webSvcApps/EGrantsSOAPApplicationSchema20030826.xsd";
	
	// Intended Processor use
	public static final int OTHER_SOURCE = 1;
	public static final int INPUT_SOURCE = 2;
	public static final int TRANSFORM_SOURCE = 3;

	// Supported Context Paths
//	private static final String CONTEXT_PATH = "gov.grants.grantapplication:gov.grants.attachments:gov.grants.sf424:gov.grants.sf424a:gov.grants.sf424b:gov.grants.sf424c:gov.grants.sf424d:gov.grants.system";
	public static final String GRANT_APPLICATION_CONTEXT_PATH = "gov.grants.grantapplication"; // used if dealing with just the application XML
	public static final String SOAP_CONTEXT_PATH = "org.xmlsoap.schemas.soap.envelope"; // used if dealing with entire SOAP Envelope
	
	private Unmarshaller unmarshaller;
	private UnmarshallerHandler unmarshallerHandler;
	private JAXBContext jaxbContext;
	private SAXParserFactory saxParserFactory;
	private XMLReader xmlReader;
	private File applicationFile;
	private gov.grants.grantapplication.GrantApplication coreGrantApplication; // used if dealing with just the application XML
	private gov.grants.grantapplication.GrantApplicationType coreGrantApplicationType; // used if dealing with entire SOAP Envelope
	private org.xmlsoap.schemas.soap.envelope.Envelope soapEnvelope; // used if dealing with entire SOAP Envelope
	
	// CONSTRUCTORS
/*
 *  GrantApplicationProcessor()
 */
	public GrantApplicationProcessor( int processorType, String contextPath ) {
		try {
			
			switch( processorType ) {
				case OTHER_SOURCE:
					System.out.println( "Creating JAXB Context" );
					jaxbContext = createContext( contextPath );
					System.out.println( "Done Creating JAXB Context" );
					System.out.println( "Creating JAXB Unmarshaller" );
					unmarshaller = createUnmarshaller( jaxbContext );
					System.out.println( "Done Creating JAXB Unmarshaller" );
					break;
				case INPUT_SOURCE:
					// Use the following if you plan on unmarshalling from an
					// org.xml.sax.InputSource. I.e., if you plan on using the
					// unmarshal( org.xml.sax.InputSource inputSource ) method and 
					// work with the entire SOAP Envelope
					System.out.println( "Creating JAXB Context" );
					jaxbContext = createContext( contextPath );
					System.out.println( "Done Creating JAXB Context" );
					System.out.println( "Creating JAXB Unmarshaller" );
					unmarshaller = createUnmarshaller( jaxbContext );
					System.out.println( "Done Creating JAXB Unmarshaller" );
					System.out.println( "Creating JAXB UnmarshallerHandler" );
					unmarshallerHandler = unmarshaller.getUnmarshallerHandler();
					System.out.println( "Done Creating JAXB UnmarshallerHandler" );
					System.out.println( "Creating JAXB UnmarshallerHandler" );
					unmarshallerHandler = unmarshaller.getUnmarshallerHandler();
					System.out.println( "Done Creating JAXB UnmarshallerHandler" );
					System.out.println( "Creating SAXParserFactory" );
					saxParserFactory = createSAXParserFactory();
					System.out.println( "Done Creating SAXParserFactory" );
					System.out.println( "Creating XMLReader" );
					xmlReader = saxParserFactory.newSAXParser().getXMLReader();
					xmlReader.setContentHandler( unmarshallerHandler );
					System.out.println( "Done Creating XMLReader" );
					break;
				case TRANSFORM_SOURCE:
					System.setProperty( "javax.xml.parsers.SAXParserFactory", "org.apache.xerces.jaxp.SAXParserFactoryImpl" );
					GrantsGovConstantsUtils.setSystemProperties();
					
					SAXParserFactory spf = SAXParserFactory.newInstance();
					spf.setNamespaceAware(true);
					spf.setValidating( false );
					SAXParser saxParser = spf.newSAXParser();
					
					System.out.println( "saxParser=" + saxParser );
					
					try {
					   saxParser.setProperty( JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA );
					   System.out.println( "JAXP_SCHEMA_LANGUAGE property set" );
					   saxParser.setProperty( JAXP_SCHEMA_SOURCE, SOAP_ENVELOPE_SCHEMA );
					   System.out.println( "JAXP_SCHEMA_SOURCE property set" );
					} catch ( org.xml.sax.SAXNotRecognizedException x ) {
					   System.out.println( "SAXParserNotRecognized Exception caught: " + x );
					   x.printStackTrace();
					}
					
					XMLReader xmlReader = saxParser.getXMLReader();
					
					System.out.println( "xmlReader=" + xmlReader );
					
					// Setup JAXB to unmarshal
					System.out.println( "Creating JAXB Context" );
					jaxbContext = createContext( SOAP_CONTEXT_PATH );
					System.out.println( "Done Creating JAXB Context" );
					System.out.println( "Creating JAXB Unmarshaller" );
					unmarshaller = createUnmarshaller( jaxbContext );
					System.out.println( "Done Creating JAXB Unmarshaller" );
					break;
				default:
					System.out.println( "Creating JAXB Context" );
					jaxbContext = createContext( contextPath );
					System.out.println( "Done Creating JAXB Context" );
					System.out.println( "Creating JAXB Unmarshaller" );
					unmarshaller = createUnmarshaller( jaxbContext );
					System.out.println( "Done Creating JAXB Unmarshaller" );
			}// switch()
		} catch (Exception e) {
			System.out.println( "There has been a problem either creating the " +
								"context for one of the following packages: " +
								contextPath + "' or creating an unmarshaller for it" +
								".\n\nFormally, the problem is a " + e );
		}
	}
	
	
	// GETTERS

	// SF424 Core
	public gov.grants.grantapplication.GrantApplication getGrantApplication() {
		return this.coreGrantApplication;
	}
	
	public gov.grants.grantapplication.GrantApplicationType getGrantApplicationType() {
		return this.coreGrantApplicationType;
	}
	
	//
	// SETTERS
	//
	
	//	
	// UTILITY
	//
	public void resetGrantApplication() {
		this.coreGrantApplication = null;
	}

	//
	// PUBLIC METHODS
	//
	public void unmarshal( org.xml.sax.InputSource inputSource )
	throws JAXBException, ParserConfigurationException, SAXException, IOException  {
		xmlReader.parse( inputSource );
		coreGrantApplication = ( GrantApplication ) unmarshallerHandler.getResult();
		System.out.println( "Got grantapplication:GrantApplication" );
	} // unmarshal()
	
	public void unmarshal( javax.xml.transform.Source source )
	throws JAXBException, ParserConfigurationException, SAXException, IOException  {

		javax.xml.bind.util.ValidationEventCollector vec = new javax.xml.bind.util.ValidationEventCollector();
//		unmarshaller.setEventHandler( vec );
		
		// turn off the JAXB provider's default validation mechanism to 
		// avoid duplicate validation
		unmarshaller.setValidating( true );
		
		// unmarshal
		soapEnvelope = ( Envelope ) unmarshaller.unmarshal( source );
		
		// check for events
		if( vec.hasEvents() ) {
			System.out.println( "vec.hasEvents()" );
		  // iterate over events
		}
		
		coreGrantApplicationType =  ( GrantApplicationType ) soapEnvelope.getBody().getGrantApplication();
		System.out.println( "Got grantapplication:GrantApplication" );
	} // unmarshal()
	
	public void unmarshal( String fileName )
	throws JAXBException, ParserConfigurationException, SAXException, IOException, Exception {
		coreGrantApplication = ( GrantApplication ) unmarshaller.unmarshal( createApplication( fileName ) );
		System.out.println( "Got grantapplication:GrantApplication" );
	} // unmarshal()
	
	public void unmarshal( File file )
	throws JAXBException, ParserConfigurationException, SAXException, IOException, Exception {
		coreGrantApplication = ( GrantApplication ) unmarshaller.unmarshal( file );
		System.out.println( "Got grantapplication:GrantApplication" );
	} // unmarshal()

	public void unmarshal( java.io.InputStream inputStream )
	throws JAXBException, ParserConfigurationException, SAXException, IOException  {
		coreGrantApplication = ( GrantApplication ) unmarshaller.unmarshal( inputStream );
		System.out.println( "Got grantapplication:GrantApplication" );
	} // unmarshal()
	
	public void unmarshal( org.w3c.dom.Node node )
	throws JAXBException, ParserConfigurationException, SAXException, IOException  {
		coreGrantApplication = ( GrantApplication ) unmarshaller.unmarshal( node );
		System.out.println( "Got core:GrantApplication" );
	} // unmarshal()
	
	public void unmarshal( Document applicationDoc )
	throws JAXBException, ParserConfigurationException, SAXException, IOException  {
		coreGrantApplication = ( GrantApplication ) unmarshaller.unmarshal( applicationDoc );
		System.out.println( "Got core:GrantApplication" );
	} // unmarshal()
	
	// PRIVATE METHODS
	private Document createApplication( String xmlFileName ) throws Exception {
		try {
			
			applicationFile = createFile( xmlFileName );
			
			// If desired, can plug in some other XML parser here, 
			// such as SAX or JDOM, if wish to avoid use of Java 
			// XML Binding (JAXB) techniques
			return parse( applicationFile );
			
		} catch (JAXBException e) {
			System.out.println( "There has been a problem either creating the " +
								"context for one of the packages, creating an " +
								"unmarshaller for it, or unmarshalling the '" +
								xmlFileName + "' file.\n\nFormally, the problem is a " + e );
			throw new Exception( "There has been a problem either creating the " +
								"context for one of the packages, creating an " +
								"unmarshaller for it, or unmarshalling the '" +
								xmlFileName + "' file.<br><br>Formally, the problem is a " + e );
								
		} catch ( FileNotFoundException e ) {
			System.out.println("There has been a problem locating the '" +
			xmlFileName + "' file.\n\nFormally, the problem is a " + e );
			throw new Exception( "There has been a problem locating the '" +
			xmlFileName + "' file.<br><br>Formally, the problem is a " + e );
		} catch ( Exception e ) {
			System.out.println("There has been a problem. Formally, the problem is a " + e );
			throw new Exception( "There has been a problem. Formally, the problem is a " + e ); 
		}
	}
	
	private File createFile( String xmlFileName ) throws FileNotFoundException {
		File file = new File( xmlFileName );
		if ( !file.exists() ) { 
			throw new FileNotFoundException();
		}
		return file;
	}
	
	private Document parse( File applicationFile )
	throws JAXBException, ParserConfigurationException, SAXException, IOException  {
		
		DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
		docBuilderFactory.setNamespaceAware(true);
		DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
		Document applicationDoc = docBuilder.parse( applicationFile );
		System.out.println( "Got applicationDoc" );
		return applicationDoc;
	} // parse()
	
   /** Note: to be really useful, an appropiate toString() method
	 * should be added to the JAXB generated objects to be debugged;
	 * i.e., add a relevant toString() method to the desired
	 * *TypeImpl classes defined in the gov.grants.*.impl folders;
     */ 
	private void debugApplication() {
		System.out.println( coreGrantApplication );
	}
	
	private static JAXBContext createContext( String contextPath ) throws JAXBException {
		return JAXBContext.newInstance( contextPath );
	}
	
	private static Unmarshaller createUnmarshaller( JAXBContext jaxbContext) throws JAXBException {
		Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
		unmarshaller.setValidating( true );
		return unmarshaller;
	}
	
	private static SAXParserFactory createSAXParserFactory() throws JAXBException {
		SAXParserFactory spf = SAXParserFactory.newInstance();
		spf.setNamespaceAware( true );
		return spf;
	}
	
   /**
     * Use for 'batch', or standalone processing
     */
	public static void main( String[] args ) {
		String xmlFilename = "";
		GrantApplicationProcessor processor;
		try {
			if ( args[0] != null ) {
				xmlFilename = args[0];
				processor = new GrantApplicationProcessor( GrantApplicationProcessor.OTHER_SOURCE, GrantApplicationProcessor.GRANT_APPLICATION_CONTEXT_PATH );
				processor.unmarshal( xmlFilename );
			} else {
				System.out.println( "USAGE:  java GrantApplicationProcessor xmlFileName" );
			}
		} catch ( Exception e ) {
			System.out.println( "There has been a problem starting up the " +
						"GrantApplicationProcessor using filename " + 
						xmlFilename + 
						".\n\nFormally, the problem is a " + e );
		}
	}
}
