/*
 * $Id: GetApplicationListReceivingServlet.java,v 1.7 2003/09/15 16:38:40 grantsuser Exp $
 * $Revision: 1.7 $
 * $Date: 2003/09/15 16:38:40 $
 */

package gov.grants.webSvcs.servlet.receiver;

// Sun
import javax.xml.soap.*;
import javax.xml.messaging.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.xml.transform.*;
import javax.naming.*;

import java.io.InputStream;

// jwsdp - sun
import gov.grants.system.GrantSubmissionFooter;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;
import javax.xml.parsers.DocumentBuilder;
import gov.grants.system.GrantSubmissionFooter;
import javax.xml.parsers.DocumentBuilderFactory;
//import javax.xml.parsers.ParserConfigurationException;

// Grants.gov
import gov.grants.utility.GrantsGovConstantsUtils;
import gov.grants.utility.GrantsGovConstants;
import gov.grants.webSvcs.utility.GrantsGovWebSvcsMediator;
import gov.grants.webSvcs.utility.GrantsGovWebSvcsMediatorStub;
import gov.grants.webSvcs.valueobject.*;
import gov.grants.webSvcs.utility.*;
import gov.grants.system.GrantSubmissionFooter;
import gov.grants.system.GrantSubmissionHeader;
import gov.grants.utility.DateUtils;

/**
 * Servlet that receives a getApplication() message, and returns
 * an application.
 *
 * Based on the sample servlet, ReceivingServlet.java, provided in
 * jaxm-simple.war
 *
 * @david xia (dxia@att.com)
 * @modified by Mark Sommer 9-12-2003
 */
public class GetApplicationListReceivingServlet extends JAXMServlet implements ReqRespListener {

	private static MessageFactory messageFactory = null;
	private static DocumentBuilderFactory docBuilderFactory;
	private static DocumentBuilder docBuilder;

	private static String appPath = null;
	private static ServletContext context;
	private static InputStream in;
	private static SOAPMessage responseMsg;
	private static GrantsGovWebSvcsMediator grantsGovWebSvcsMediator;

	static {
        try {
			GrantsGovConstantsUtils.setSystemProperties();
            messageFactory = MessageFactory.newInstance();
        } catch (Exception e) {
            e.printStackTrace();
        }
    };

    public void init(ServletConfig servletConfig) throws ServletException {
        super.init(servletConfig);

		try {
            context = getServletContext();
            System.out.println( "context=" + context );
            appPath = GrantsGovConstantsUtils.getApplicationPath( context.getRealPath( "/" ) );// application path
            System.out.println( "appPath= " + appPath );

            docBuilderFactory = DocumentBuilderFactory.newInstance();
            docBuilderFactory.setNamespaceAware(true);
            docBuilder = docBuilderFactory.newDocumentBuilder();

            grantsGovWebSvcsMediator = new GrantsGovWebSvcsMediatorStub( context );

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

	//
    // This is the application code for handling the message.
	//
    public SOAPMessage onMessage(SOAPMessage requestMessage) {
        System.out.println("On message called in receiving servlet");
        try {
                System.out.println( "Here's the request message: " );
                requestMessage.writeTo(System.out);
				System.out.println( "\n\n" );

                SOAPMessage returnMessage = null;
                //GetApplicationListReceivedMessage receivedMessage = new GetApplicationListReceivedMessage(requestMessage);

                //Get Filter Type
                java.lang.String filterType=SOAPUtils.getSOAPElement( requestMessage, "FilterType", "GetApplicationList" );

                //Get Filter Value
                java.lang.String filterValue=SOAPUtils.getSOAPElement( requestMessage, "FilterValue", "GetApplicationList" );

                //Since we are not really generating from the database, we just
                //output the parsed SOAP paraments for demo purpose.
                System.out.println( "FilterType = " + filterType );
                System.out.println( "FilterValue = " + filterValue );

                // Get the Application List
                java.util.List applicationList = grantsGovWebSvcsMediator.getApplicationList( filterType, filterValue );

				// create the response
                responseMsg = createResponseMsg( applicationList );

                return responseMsg;

        } catch(Exception e) {
            System.out.println("Error in processing or replying to a message:  " + e);
			try {
				responseMsg = SOAPUtils.generateSOAPFaultMsg( messageFactory,
															  "Error in processing or replying to a message:  " + e,
															  GrantsGovConstants.LOCAL_NAME_SERVER );
				return responseMsg;
			} catch( Exception ex ) {
				System.out.println( "Error trying to generate a SOAP Fault Message:  " + ex );
				return null;
			}
        }
    }

	private SOAPMessage createResponseMsg( java.util.List applicationList)
	throws javax.xml.soap.SOAPException, org.xml.sax.SAXException, java.io.IOException {

		System.out.println( "In createResponseMsg" );
		SOAPMessage responseMsg = messageFactory.createMessage();
		System.out.println( "Response Msg Created; responseMsg =" + responseMsg );
		SOAPEnvelope soapEnvelope = responseMsg.getSOAPPart().getEnvelope();
		SOAPBody responseBody = soapEnvelope.getBody();
		SOAPElement rootElement = responseBody.addChildElement(soapEnvelope.createName( "GetApplicationListResponse", GrantsGovConstants.AGIS_PREFIX, GrantsGovConstants.AGIS_URI ));
		SOAPElement applicationInformationElement;
		SOAPElement statusElement;
		SOAPElement paramElement;
		GrantSubmissionFooter footer;
		GrantSubmissionHeader header;

		ApplicationInformation applicationInformation;
		int count = applicationList.size();

		paramElement = rootElement.addChildElement("AvailableApplicationNumber");
		paramElement.addTextNode(Integer.toString( count ));

		java.util.ListIterator it = applicationList.listIterator();
		while (it.hasNext()){
			applicationInformation = (ApplicationInformation)it.next();

			footer = applicationInformation.getGrantSubmissionFooter();
			header = applicationInformation.getGrantSubmissionHeader();
			applicationInformationElement = rootElement.addChildElement(soapEnvelope.createName( "ApplicationInformation" ));
			
			statusElement = applicationInformationElement.addChildElement(soapEnvelope.createName( "Status" ));
			statusElement.addTextNode( applicationInformation.getStatus() );

			//add header info
			addNotNullElement(applicationInformationElement, "CFDANumber", header.getCFDANumber());
			addNotNullElement(applicationInformationElement, "OpportunityID", header.getOpportunityID());
			addNotNullElement(applicationInformationElement, "CompetitionID", header.getCompetitionID());

			//add footer info
			addNotNullElement(applicationInformationElement, "ReceivedDateTime", DateUtils.getDateAsXSDDate( footer.getReceivedDateTime() ));
			addNotNullElement(applicationInformationElement, "Grants_govTrackingNumber", footer.getGrantsGovTrackingNumber() );

		}

		responseMsg.saveChanges();

		System.out.println( "Finished Creating Response Message" );

		responseMsg.writeTo(System.out);

		return responseMsg;

	}// createResponseMsg()

	public void addNotNullElement(SOAPElement parentElement, String elementName, Object object)
	throws SOAPException {
		SOAPElement childElement;
		if (object != null)
		{
			System.out.println( "Adding " + elementName + " = " + object.toString() );
			childElement = parentElement.addChildElement(elementName);
			childElement.addTextNode(object.toString());
		}
	}
}

