/*
 * $Id: AssignAgencyTrackingNumberSendingServlet.java,v 1.5 2003/09/15 16:39:06 grantsuser Exp $
 * $Revision: 1.5 $
 * $Date: 2003/09/15 16:39:06 $
 */

package gov.grants.webSvcs.servlet.sender;

// sun
import java.net.*;
import java.io.*;
import java.util.*;

import javax.servlet.http.*;
import javax.servlet.*;

import javax.xml.soap.*;

import javax.activation.*;
import javax.naming.*;

// Grants.gov
import gov.grants.utility.GrantsGovConstantsUtils;
import gov.grants.utility.GrantsGovConstants;
import gov.grants.webSvcs.utility.SOAPUtils;

/**
 * Servlet that is used for sending a 'AssignAgencyTrackingNumber' request message.
 *
 * Based on the sample servlet, SendingServlet.java, provided in
 * jaxm-simple.war
 *
 * @author Mark Sommer
 */
public class AssignAgencyTrackingNumberSendingServlet extends HttpServlet {

    private String recipient = null;
    private String attachmentData = null;
    private ServletContext servletContext;
	private RequestDispatcher rd;

    // Connection to send messages.
    private SOAPConnection con;

    public void init(ServletConfig servletConfig) throws ServletException {
		GrantsGovConstantsUtils.setSystemProperties();
        super.init( servletConfig );
        servletContext = servletConfig.getServletContext();

        try {
			SOAPConnectionFactory scf = SOAPConnectionFactory.newInstance();
            con = scf.createConnection();
        } catch(Exception e) {
            System.out.println("Unable to open a SOAPConnection" + e);
        }

        InputStream in = servletContext.getResourceAsStream( GrantsGovConstants.APPLICATION_PROPERTIES );

        if ( in != null ) {
            Properties props = new Properties();

            try {
                props.load(in);

                recipient = props.getProperty("AssignAgencyTrackingNumberRecipient");
				System.out.println( "RECIPIENT has been loaded from properties file, and is set to " + recipient );
            } catch (IOException ex) {
                // Ignore
            }
        }
    }

    public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException,ServletException	{
		doPost( request, response );
	}

	public void doPost(HttpServletRequest request, HttpServletResponse response ) throws IOException,ServletException	{

		String grants_govTrackingNumber = request.getParameter( "grants_govTrackingNumber" );
		System.out.println( "grants_govTrackingNumber = " + grants_govTrackingNumber );

		String agencyTrackingNumber = request.getParameter( "agencyTrackingNumber" );
		System.out.println( "agencyTrackingNumber = " + agencyTrackingNumber );

        try {
            // Create a message factory.
            MessageFactory mf = MessageFactory.newInstance();

            // Create a message from the message factory.
            SOAPMessage msg = mf.createMessage();

            // Message creation takes care of creating the SOAPPart - a
            // required part of the message as per the SOAP 1.1
            // specification.
            SOAPPart sp = msg.getSOAPPart();

            // Retrieve the envelope from the soap part to start building
            // the soap message.
            SOAPEnvelope envelope = sp.getEnvelope();

            // Create a soap header from the envelope.
            SOAPHeader hdr = envelope.getHeader();

            // Create a soap body from the envelope.
            SOAPBody bdy = envelope.getBody();

            // Add a soap body element to the soap body
			SOAPBodyElement requestElement;
			requestElement = bdy.addBodyElement(envelope.createName("AssignAgencyTrackingNumber", GrantsGovConstants.AGIS_PREFIX, GrantsGovConstants.AGIS_URI ));
			System.out.println( "SOAPBodyElement =" + requestElement.getClass().getName() );
			requestElement.addChildElement(envelope.createName("Grants_govTrackingNumber")).addTextNode(grants_govTrackingNumber);
			requestElement.addChildElement(envelope.createName("AgencyTrackingNumber")).addTextNode(agencyTrackingNumber);

            StringBuffer urlSB = new StringBuffer();
            urlSB.append(request.getScheme()).append("://").append(request.getServerName());
            urlSB.append( ":" ).append( request.getServerPort() ).append( request.getContextPath() );
            String requestBase=urlSB.toString();

			System.out.println( "requestBase = " + requestBase );

            // Create an endpoint for the recipient of the message.
            if( recipient == null ) {
                recipient=requestBase + "/AgencyIntegration/AssignAgencyTrackingNumber";
				System.out.println( "AssignAgencyTrackingNumber recipient was null: Has been set to " + recipient );
            }

            URL urlEndpoint = new URL(recipient);

            System.out.println("Sending message to URL: " + urlEndpoint);
            System.out.println("Sent message is logged in \"AssignAgencyTrackingNumberRequestMessage.txt\"");

            FileOutputStream sentFile = new FileOutputStream("AssignAgencyTrackingNumberRequestMessage.txt");
            msg.writeTo(sentFile);
            sentFile.close();

            // Send the message to the provider using the connection.
			System.out.println( "About to send request message" );
            SOAPMessage replyMessage = con.call(msg, urlEndpoint);
			System.out.println( "Received response message;" );

            if (replyMessage != null) {

				// Write out the replyMessage SOAP Message to File system as well for debug purposes
                FileOutputStream replyFile = new FileOutputStream("AssignAgencyTrackingNumberResponseMessage.txt");
                replyMessage.writeTo(replyFile);
                replyFile.close();
                System.out.println( "Reply logged in \"AssignAgencyTrackingNumberResponseMessage.txt\"" );
				
				// Check for, process SOAP Fault
				SOAPBody replyBody = replyMessage.getSOAPPart().getEnvelope().getBody();
				if ( replyBody.hasFault() ) {
					System.out.println( "SOAP message has Fault; processing Fault" );
				 	SOAPUtils.processSOAPFault( replyBody );
				} else {
					System.out.println( "No Faults" );
				}
				
            } else {
                System.out.println("No reply was received");
            }

        } catch(Throwable e) {
            e.printStackTrace();
            System.out.println("Error in constructing or sending message " + e.getMessage());

			request.setAttribute( "ErrorMsg", e.getMessage() );

			System.out.println( "About to forward to /xmlToDB/jsp/include/xmlToDBErrorPage.jsp" );
			rd = request.getRequestDispatcher("/xmlToDB/jsp/include/xmlToDBErrorPage.jsp");
			rd.forward(request, response);
			return;
        }

		System.out.println( "About to forward to /webSvcs/jsp/WebServices.jsp" );
		rd = request.getRequestDispatcher("/webSvcs/jsp/WebServices.jsp");
		rd.forward(request, response);
    }

}
