/*
 * $Id: GetApplicationListSendingServlet.java,v 1.8 2003/09/16 21:08:44 cvsuser Exp $
 * $Revision: 1.8 $
 * $Date: 2003/09/16 21:08:44 $
 */

package gov.grants.webSvcs.servlet.sender;

// sun
import java.net.*;
import java.io.*;
import java.util.*;

import javax.servlet.http.*;
import javax.servlet.*;

import javax.xml.soap.*;

import javax.activation.*;
import javax.naming.*;

// Grants.gov
import gov.grants.webSvcs.valueobject.ApplicationListInformation;
import gov.grants.system.*;
import gov.grants.utility.GrantsGovConstantsUtils;
import gov.grants.utility.GrantsGovConstants;
import gov.grants.webSvcs.utility.SOAPUtils;

/**
 * Servlet that is used for sending a getApplication() request message.
 *
 * Based on the sample servlet, GetApplicationSendingServlet.java, provided in 
 * jaxm-simple.war 
 *
 * @david xia */
public class GetApplicationListSendingServlet extends HttpServlet {

    String recipient = null;
    String attachmentData = null;
	String appPath = null;
    ServletContext servletContext;

    // Connection to send messages.
    private SOAPConnection con;

    public void init(ServletConfig servletConfig) throws ServletException {
		
		GrantsGovConstantsUtils.setSystemProperties();
//		System.getProperties().list( System.out );
		
        super.init( servletConfig );
        servletContext = servletConfig.getServletContext();
		System.out.println( "servletContext=" + servletContext );
		
//		appPath = GrantsGovConstantsUtils.getApplicationPath( servletContext.getRealPath( "/" ) );// application path
//		System.out.println( "appPath= " + appPath );

		System.out.println( "Loading SOAPConnectionFactory" );

        try {
			SOAPConnectionFactory scf = SOAPConnectionFactory.newInstance();
			con = scf.createConnection();
        } catch( Exception e ) {
            System.out.println("Unable to open a SOAPConnection" + e);
        }

//		String appPropsPath =  appPath + 
//							"WEB-INF" +
//							System.getProperty( "file.separator" ) + 
//							"application.properties";
//		System.out.println( "appPropsPath=" + appPropsPath );
        InputStream in = servletContext.getResourceAsStream( GrantsGovConstants.APPLICATION_PROPERTIES );

        if ( in != null ) {
            Properties props = new Properties();

            try {
                    props.load(in);
                    recipient = props.getProperty("GetApplicationListRecipient");
                    System.out.println( "RECIPIENT has been loaded from properties file, and is set to " + recipient );
            } catch (IOException ex) {
                // Ignore
            }
        } else {
			System.out.println( "Error Loading /WEB-INF/application.properties" );
		}
    }
	
    public void doGet( HttpServletRequest request, HttpServletResponse response) throws IOException,ServletException	{
		doPost( request, response );
	}
	
	public void doPost( HttpServletRequest request, HttpServletResponse response ) throws IOException,ServletException	{

        java.util.List  applicationList;
		RequestDispatcher rd;

        try {
			

            // Create a message factory.
            MessageFactory mf = MessageFactory.newInstance();
			System.out.println( "MessageFactory =" + mf.getClass().getName() );

            // Create a message from the message factory.
            SOAPMessage msg = mf.createMessage();
			System.out.println( "SOAPMessage =" + msg.getClass().getName() );

            // Message creation takes care of creating the SOAPPart - a
            // required part of the message as per the SOAP 1.1
            // specification.
            SOAPPart sp = msg.getSOAPPart();
			System.out.println( "SOAPPart =" + sp.getClass().getName() );

            // Retrieve the envelope from the soap part to start building
            // the soap message.
            SOAPEnvelope envelope = sp.getEnvelope();
			System.out.println( "SOAPEnvelope =" + envelope.getClass().getName() );

            // Create a soap header from the envelope.
            SOAPHeader hdr = envelope.getHeader();
			System.out.println( "SOAPHeader =" + hdr.getClass().getName() );

            // Create a soap body from the envelope.
            SOAPBody bdy = envelope.getBody();
			System.out.println( "SOAPBody =" + bdy.getClass().getName() );

            // Add a soap body element to the soap body
            SOAPBodyElement requestElement;
            requestElement = bdy.addBodyElement(envelope.createName( "GetApplicationList", GrantsGovConstants.AGIS_PREFIX, GrantsGovConstants.AGIS_URI ));
			System.out.println( "SOAPBodyElement =" + requestElement.getClass().getName() );
			SOAPElement paramElement1, paramElement2;
            paramElement1 = requestElement.addChildElement("FilterType");
            paramElement1.addTextNode("Status");
			System.out.println( "SOAPElement =" + paramElement1.getClass().getName() );
            paramElement2 = requestElement.addChildElement("FilterValue");
            paramElement2.addTextNode("All");
            
            StringBuffer urlSB = new StringBuffer();
            urlSB.append(request.getScheme()).append("://").append(request.getServerName());
            urlSB.append( ":" ).append( request.getServerPort() ).append( request.getContextPath() );
            String requestBase=urlSB.toString();

            System.out.println( "requestBase = " + requestBase );

            // Create an endpoint for the recipient of the message.
            if( recipient == null ) {
                recipient=requestBase + "/AgencyIntegration/GetApplicationList";
				System.out.println( "GetApplicationList recipient was null: Has been set to " + recipient );
            }

            URL urlEndpoint = new URL(recipient);

            System.out.println("Sending message to URL: " + urlEndpoint);
            System.out.println("Sent message is logged in \"GetApplicationListRequestMessage.txt\"");

            FileOutputStream sentFile = new FileOutputStream("GetApplicationListRequestMessage.txt");
            msg.writeTo(sentFile);
            sentFile.close();

            // Send the message to the provider using the connection.
			SOAPConnectionFactory scf = SOAPConnectionFactory.newInstance();
			System.out.println( "Loading SOAPConnectionFactory; scf =" + scf );
			con = scf.createConnection();
            SOAPMessage replyMessage = con.call(msg, urlEndpoint);

            if ( replyMessage != null ) {
				
				// log reply message
                FileOutputStream replyFile = new FileOutputStream( "GetApplicationListResponseMessage.txt" );
                replyMessage.writeTo(replyFile);
                replyFile.close();
                System.out.println( "Reply logged in \"GetApplicationListResponseMessage.txt\"" );
				
				// Check for, process SOAP Fault
				SOAPBody replyBody = replyMessage.getSOAPPart().getEnvelope().getBody();
				if ( replyBody.hasFault() ) {
					System.out.println( "SOAP message has Fault; processing Fault" );
				 	SOAPUtils.processSOAPFault( replyBody );
				} else {
					System.out.println( "No Faults; about to retrieve application list" );
					applicationList = retrieveApplicationList( replyMessage );
					request.setAttribute( "ApplicationList", applicationList );
				}

            } else {
                System.out.println( "No Reply" );
            }

        } catch( Throwable e ) {
            e.printStackTrace();
            System.out.println("Error in constructing or sending message " + e.getMessage());

			request.setAttribute( "ErrorMsg", e.getMessage() );
			
			System.out.println( "About to forward to /xmlToDB/jsp/include/xmlToDBErrorPage.jsp" );
			rd = request.getRequestDispatcher( "/xmlToDB/jsp/include/xmlToDBErrorPage.jsp" );
			rd.forward(request, response);
			return;
        }

		System.out.println( "About to forward to /webSvcs/jsp/WebServices.jsp" );
		rd = request.getRequestDispatcher("/webSvcs/jsp/WebServices.jsp");
		rd.forward(request, response);
    }
	
    public List retrieveApplicationList(SOAPMessage replyMsg)
	throws javax.xml.soap.SOAPException, javax.xml.bind.JAXBException, org.xml.sax.SAXException, java.io.IOException {
 
 		System.out.println( "Retrieving ApplicationList" );

        ArrayList applicationList = new ArrayList();
        ApplicationListInformation applicationListInformation;
        String status;
        int count;
        
        SOAPPart soapPart;
        SOAPEnvelope soapEnvelope;
        SOAPBody soapBody;
        SOAPElement rootElement;
        SOAPElement footer;
        SOAPElement header;
        
        javax.xml.soap.Name functionName;
        javax.xml.soap.Name applicationNumberName;
        javax.xml.soap.Name applicationInformationName;

        soapPart =replyMsg.getSOAPPart();
        soapEnvelope = soapPart.getEnvelope();
        soapBody = soapEnvelope.getBody();
       
        // Search for Function name
        functionName = soapEnvelope.createName( "GetApplicationListResponse", GrantsGovConstants.AGIS_PREFIX, GrantsGovConstants.AGIS_URI );
        applicationNumberName = soapEnvelope.createName( "AvailableApplicationNumber" );
        applicationInformationName = soapEnvelope.createName( "ApplicationInformation" );
        
        java.util.Iterator valueRequests = soapBody.getChildElements(functionName);
  
        //get available applicatin number
        rootElement = (SOAPElement)valueRequests.next();
        SOAPElement param = (SOAPElement)(rootElement.getChildElements(applicationNumberName).next());
    
        count = Integer.parseInt(param.getValue());
        System.out.println( "AvailableApplicationNumber = " + param.getValue() );
        
        java.util.Iterator applicationListInformationIterator = rootElement.getChildElements( applicationInformationName );

        //get application list information
        for (int i=0; i<count; i++){
			
            param = (SOAPElement)(applicationListInformationIterator.next());
 
            applicationListInformation = new ApplicationListInformation();
			
			if ((param.getChildElements(soapEnvelope.createName("OpportunityID"))).hasNext()){
				applicationListInformation.setOpportunityID(((SOAPElement)param.getChildElements(soapEnvelope.createName("OpportunityID")).next()).getValue());
			}
			if ((param.getChildElements(soapEnvelope.createName("CompetitionID"))).hasNext()){
				applicationListInformation.setCompetitionID(((SOAPElement)param.getChildElements(soapEnvelope.createName("CompetitionID")).next()).getValue());
			}
			if ((param.getChildElements(soapEnvelope.createName("CFDANumber"))).hasNext()){
				applicationListInformation.setCFDANumber(((SOAPElement)param.getChildElements(soapEnvelope.createName("CFDANumber")).next()).getValue());
			}
			if ((param.getChildElements(soapEnvelope.createName("Grants_govTrackingNumber"))).hasNext()){
				applicationListInformation.setGrantsGovTrackingNumber(((SOAPElement)param.getChildElements(soapEnvelope.createName("Grants_govTrackingNumber")).next()).getValue());
			}
			if ((param.getChildElements(soapEnvelope.createName("ReceivedDateTime"))).hasNext()){
				applicationListInformation.setReceivedDateTime(((SOAPElement)param.getChildElements(soapEnvelope.createName("ReceivedDateTime")).next()).getValue());
			}

			status = ((SOAPElement)(param.getChildElements(soapEnvelope.createName("Status")).next())).getValue();
			applicationListInformation.setStatus(status);
			
			applicationList.add(applicationListInformation);
		}

          
        return applicationList;
    }

}
