/*
 * $Id: GetApplicationSendingServlet.java,v 1.16 2003/09/16 21:08:44 cvsuser Exp $
 * $Revision: 1.16 $
 * $Date: 2003/09/16 21:08:44 $
 */

package gov.grants.webSvcs.servlet.sender;

// sun
import java.net.*;
import java.io.*;
import java.util.*;

import javax.servlet.http.*;
import javax.servlet.*;

import javax.xml.soap.*;

import javax.activation.*;
import javax.naming.*;

// Grants.gov
import gov.grants.utility.GrantsGovConstantsUtils;
import gov.grants.utility.GrantsGovConstants;
import gov.grants.utility.GrantApplicationProcessor;
import gov.grants.xmlToDB.utility.DaoMediator;
import gov.grants.webSvcs.utility.SOAPUtils;


/**
 * Servlet that is used for sending a 'GetApplication' request message.
 *
 * Based on the sample servlet, GetApplicationSendingServlet.java, provided in
 * jaxm-simple.war
 *
 * @author Dan Dumitru
 */
public class GetApplicationSendingServlet extends HttpServlet {

    private String recipient = null;
    private String attachmentData = null;
    private ServletContext servletContext;
	private GrantApplicationProcessor processor;
	private RequestDispatcher rd;

    // Connection to send messages.
    private SOAPConnection con;

    public void init(ServletConfig servletConfig) throws ServletException {
		GrantsGovConstantsUtils.setSystemProperties();
        super.init( servletConfig );
        servletContext = servletConfig.getServletContext();

        try {
			SOAPConnectionFactory scf = SOAPConnectionFactory.newInstance();
            con = scf.createConnection();
        } catch(Exception e) {
            System.out.println("Unable to open a SOAPConnection" + e);
        }

        InputStream in = servletContext.getResourceAsStream( GrantsGovConstants.APPLICATION_PROPERTIES );

        if ( in != null ) {
            Properties props = new Properties();

            try {
                props.load(in);

                recipient = props.getProperty("GetApplicationRecipient");
				System.out.println( "RECIPIENT has been loaded from properties file, and is set to " + recipient );
            } catch (IOException ex) {
                // Ignore
            }
        }
    }

    public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException,ServletException	{
		doPost( request, response );
	}

	public void doPost(HttpServletRequest request, HttpServletResponse response ) throws IOException,ServletException	{

		String grants_govTrackingNumber = request.getParameter( "grants_govTrackingNumber" );
		System.out.println( "grants_govTrackingNumber = " + grants_govTrackingNumber );

        try {
            // Create a message factory.
            MessageFactory mf = MessageFactory.newInstance();

            // Create a message from the message factory.
            SOAPMessage msg = mf.createMessage();

            // Message creation takes care of creating the SOAPPart - a
            // required part of the message as per the SOAP 1.1
            // specification.
            SOAPPart sp = msg.getSOAPPart();

            // Retrieve the envelope from the soap part to start building
            // the soap message.
            SOAPEnvelope envelope = sp.getEnvelope();

            // Create a soap header from the envelope.
            SOAPHeader hdr = envelope.getHeader();

            // Create a soap body from the envelope.
            SOAPBody bdy = envelope.getBody();

            // Add a soap body element to the soap body
			SOAPBodyElement requestElement;
			requestElement = bdy.addBodyElement(envelope.createName("GetApplication", GrantsGovConstants.AGIS_PREFIX, GrantsGovConstants.AGIS_URI ));
			System.out.println( "SOAPBodyElement =" + requestElement.getClass().getName() );
			requestElement.addChildElement(envelope.createName("Grants_govTrackingNumber")).addTextNode(grants_govTrackingNumber);

            StringBuffer urlSB = new StringBuffer();
            urlSB.append(request.getScheme()).append("://").append(request.getServerName());
            urlSB.append( ":" ).append( request.getServerPort() ).append( request.getContextPath() );
            String requestBase=urlSB.toString();

			System.out.println( "requestBase = " + requestBase );

            // Create an endpoint for the recipient of the message.
            if( recipient == null ) {
                recipient=requestBase + "/AgencyIntegration/GetApplication";
				System.out.println( "GetApplication recipient was null: Has been set to " + recipient );
            }

            URL urlEndpoint = new URL(recipient);

            System.out.println("Sending message to URL: " + urlEndpoint);
            System.out.println("Sent message is logged in \"GetApplicationRequestMessage.txt\"");

            FileOutputStream sentFile = new FileOutputStream("GetApplicationRequestMessage.txt");
            msg.writeTo(sentFile);
            sentFile.close();

            // Send the message to the provider using the connection.
			System.out.println( "About to send request message" );
            SOAPMessage replyMessage = con.call( msg, urlEndpoint );
			System.out.println( "Received response message;");

            if ( replyMessage != null ) {

				// Write out the reply SOAP Message to File system as well for debug purposes
				System.out.println( "About to write SOAP response to file system" );
				FileOutputStream replyFile = new FileOutputStream("GetApplicationResponseMessage.txt");
				replyMessage.writeTo( replyFile );
				replyFile.close();
				System.out.println( "Reply logged in \"GetApplicationResponseMessage.txt\"" );

				// Check for, process SOAP Fault
				SOAPBody replyBody = replyMessage.getSOAPPart().getEnvelope().getBody();
				if ( replyBody.hasFault() ) {
					System.out.println( "SOAP message has Fault, processing Fault");
				 	SOAPUtils.processSOAPFault(replyBody);
				} else{
					System.out.println( "No Faults; about to start unmarshalling XML to Java objects" );

					// Unmarshal XML file to JAXB generated java objects

					// Use the getContent() method to retrieve the entire SOAP message as a javax.xml.transform.Source object.
					// This is useful if you want to parse the XML Document using SAX, DOM, etc.  Note:  the resulting content
					// still has the enclosing SOAP Envelope and Body tags, as well as the payload XML content.
					//
					processor = new GrantApplicationProcessor( GrantApplicationProcessor.TRANSFORM_SOURCE, GrantApplicationProcessor.SOAP_CONTEXT_PATH );
					processor.unmarshal( replyMessage.getSOAPPart().getContent() );
					//
					//processor = new GrantApplicationProcessor();
					//processor.unmarshal( reply.getSOAPBody().getFirstChild() );

					// Save the unmarshalled data to the database
					System.out.println( "XML File has been parsed/unsmarshalled; about to save to database" );
					//DaoMediator.createApplication( processor.getGrantApplication() );
					DaoMediator.createApplication( processor.getGrantApplicationType() );

					// Process any attachments
					java.util.Iterator iterator = replyMessage.getAttachments();
					if ( iterator.hasNext() ) {
						AttachmentPart attachment;
						String contentId;
						String contentType;
						InputStream attachmentInputStream;
						FileOutputStream attachmentFile;
						int n, result = 0;
						byte[] b;
						while (iterator.hasNext()) {
							attachment = (AttachmentPart)iterator.next();
							contentId = attachment.getContentId();
							contentType = attachment.getContentType();
							System.out.println( "Attachment \"" + contentId + "\" has content type " + contentType );
							if ( contentType == "text/plain" ) {
								Object content = attachment.getContent();
								System.out.println("Attachment contains:\n" + content);
							}

							// Save attachment to File System
							attachmentFile = new FileOutputStream( attachment.getContentId() );
							attachmentInputStream = (InputStream)attachment.getContent();

							while (true) {
								n = attachmentInputStream.available(); // available bytes without blocking
								if ( n > 0 ) {
									b = new byte[n];
									result = attachmentInputStream.read( b );
									if ( result == -1 ) { break; }
									attachmentFile.write( b );
								} else { break; }
							} // end while

							attachmentFile.flush();
							attachmentFile.close();
						}
					}

					// Cleanup
					processor = null;

					System.out.println( "XML File has been saved to the database" );
				}

            } else {
                System.out.println("No reply was received");
            }

        } catch(Throwable e) {
            System.out.println("Error in constructing or sending message: \n" + e.getMessage());

			request.setAttribute( "ErrorMsg", e.getMessage() );

			System.out.println( "About to forward to /xmlToDB/jsp/include/xmlToDBErrorPage.jsp" );
			rd = request.getRequestDispatcher("/xmlToDB/jsp/include/xmlToDBErrorPage.jsp");
			rd.forward(request, response);
			return;
        } finally {
			; // cleanup here
		}

		System.out.println( "About to forward to /webSvcs/jsp/WebServices.jsp" );
		rd = request.getRequestDispatcher("/webSvcs/jsp/WebServices.jsp");
		rd.forward(request, response);
    }

}
